<?php

/**
 * You should not have to change this
 */
define('API_URL', 'https://fiddlemail.com/api/v1/');


/**
 * Internal flag definitions
 */
define('FIDDLEMAIL_NONE', 0);
define('FIDDLEMAIL_EXTENDED', 1);
define('FIDDLEMAIL_SCORE', 2);

/**
 * Class for querying the fiddlemail API
 *
 * @author fiddlemail.com
 * @copyright (c) 2018, fiddlemail.com
 */
abstract class FiddleMail {

	/**
	 * Query the fiddlemail api for a mail address
	 * 
	 * @param string $address
	 * @param int $flags
	 * @throws Exception
	 * @return array
	 */
	public static function queryMail($address, $flags = 0) {
		return self::query('email', $address, $flags);
	}
	
	/**
	 * Query the fiddlemail api for a phone number
	 * 
	 * @param string $number
	 * @param int $flags
	 * @throws Exception
	 * @return array
	 */
	public static function queryPhone($number, $flags = 0) {
		return self::query('phone', $number, $flags);
	}
	
	/**
	 * Query the fiddlemail api for a mail address
	 * 
	 * @param string $type phone|email
	 * @param string $query
	 * @param int $flags
	 * @throws Exception
	 * @return array
	 */
	private static function query($type, $query, $flags = 0) {
		/** check for curl extension */
		if(!function_exists('curl_init')) {
			throw new Exception('PHP curl extension is missing');
		}
		
		/** prepare api url to call */
		$url = API_URL . API_KEY . '/' . ($type == 'phone' ? 'phone' : 'email') . '/' . urlencode($query);
		
		/** prepare http content type header */
		$headers = array(
			'Content-Type' => 'application/x-json'
		);
		
		/** prepare json data to POST to the api depending on set flags */
		$json_data = array(
			'lang' => (defined('API_LANGUAGE') ? API_LANGUAGE : 'en'),
			'score' => ($flags & FIDDLEMAIL_SCORE ? true : false),
			'extended' => ($flags & FIDDLEMAIL_EXTENDED ? true : false)
		);
		
		/** init curl object and set options */
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $url);
		curl_setopt($ch, CURLOPT_POST, 1);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($json_data));
		curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
		
		/** execute api call */
		$result = curl_exec($ch);
		
		/** get http status code of the api call */
		$http_status = curl_getinfo($ch, CURLINFO_HTTP_CODE);
		
		/** check if api call succeeded */
		if($result === false) {
			throw new Exception('API CALL FAILED');
		}
		
		/** decode json data to an array */
		$data = json_decode($result, true);
		if(!$data) {
			/** json data could not be decoded (maybe invalid) */
			throw new Exception('API CALL FAILED WITH HTTP CODE ' . $http_status . '. Invalid json data received.');
		}
		
		/** we need status 200. something went wrong otherwise. */
		if($http_status != 200) {
			throw new Exception('API CALL FAILED WITH HTTP CODE ' . $http_status . '. Json data: ' . var_export($data, true));
		} elseif($data['error'] == true) {
			/** json data could be decoded but the api sent an error code. */
			throw new Exception('API CALL FAILED: ' . var_export($data, true));
		}
		
		/** return the api result array */
		return $data;
	}
}
